-----------------------------------------------------------------------------------------------
-- Client Lua Script for TextDisplay
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- TextDisplay Module Definition
-----------------------------------------------------------------------------------------------
local TextDisplay = {
	saved_settings = {},
} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function TextDisplay:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.display_index = 1
	self.is_displayed = 0

    return o
end

function TextDisplay:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- TextDisplay Handle config
-----------------------------------------------------------------------------------------------
function TextDisplay:GetDefaultConfig()
	self.settings = {
		index = 1,
		text = {
			"Text example."
			},
	}
end

function TextDisplay:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end

	local save = {}
	save.settings = self.settings
	return save
end

function TextDisplay:OnRestore(eType, save)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	if save.settings.index ~= nil then
		self.saved_settings = save
		self:LoadSavedData( save )
	end
end

function TextDisplay:LoadSavedData( save )
	self.default_settings = self.settings
	self.settings.index = save.settings.index
	self.settings.text = save.settings.text

	-- TODO: This needs to be a deep check
	for key, value in pairs(self.default_settings) do
		if self.settings[key] == nil then
			self.settings[key] = value
		end
	end
	
	-- TODO: Verify if this is necessary <-
	for key, value in pairs(self.default_settings.text) do
		if self.settings.text[key] == nil then
			self.settings.text[key] = value
		end
	end
end

-----------------------------------------------------------------------------------------------
-- TextDisplay OnLoad
-----------------------------------------------------------------------------------------------
function TextDisplay:OnLoad()
    -- load our form file
	self:GetDefaultConfig()
	self.xmlDoc = XmlDoc.CreateFromFile("TextDisplay.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- TextDisplay OnDocLoaded
-----------------------------------------------------------------------------------------------
function TextDisplay:OnDocLoaded()
	
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "TextDisplayForm", nil, self)
		self.wndMain:Move(5,575,self.wndMain:GetWidth(),self.wndMain:GetHeight())
		-- self.wndMain:FindChild("EditBox"):SetText(self.settings.text[self.display_index])
		self.wndDisplay = Apollo.LoadForm(self.xmlDoc, "FinalDisplayForm", nil, self)
		self.wndDisplay:Move(5,575,self.wndDisplay:GetWidth(),self.wndDisplay:GetHeight())

		self.previousButton = self.wndMain:FindChild("PreviousButton")
		self.nextButton = self.wndMain:FindChild("NextButton")
		self.createButton = self.wndMain:FindChild("CreateButton")
		self.deleteButton = self.wndMain:FindChild("DeleteButton")
		
	    self.wndMain:Show(false, true)
		self.wndDisplay:Show(false, true)
		self.createButton:Show(true)
		self:UpdateUI()
	
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		if self.wndDisplay == nil then
			Apollo.AddAddonErrorText(self, "Could not load the display window for some reason.")
			return
		end

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("txd", "SlashCommand", self)
		Apollo.RegisterSlashCommand("txdp", "OutputDisplay", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ToggleTextDisplayForm", "OnToggleTextDisplayForm", self)

		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- TextDisplay Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/txd"
function TextDisplay:SlashCommand()
	self.wndMain:Invoke()
end

-- on SlashCommand "/txdp"
function TextDisplay:OutputDisplay()
	local text = self.wndMain:FindChild("EditBox"):GetText()
	local strCancel = Apollo.GetString(1)
	local partyChannel = "p"
	
	-- German
	if strCancel == "Abbrechen" then
		partyChannel = "party"
	end

	-- French
	if strCancel == "Annuler" then
		partyChannel = "éq"
	end
	
	for w in text:gmatch("(%C+)") do
		ChatSystemLib.Command("/" .. partyChannel .. " " .. w)
	end
end

-----------------------------------------------------------------------------------------------
-- TextDisplayForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function TextDisplay:OnOK()
	self.settings.text[self.display_index] = self.wndMain:FindChild("EditBox"):GetText()
	self.wndDisplay:FindChild("EditBox"):SetText(self.wndMain:FindChild("EditBox"):GetText())
end

function TextDisplay:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "TextDisplay", {"ToggleTextDisplayForm", "", ""})
end

function TextDisplay:OnToggleTextDisplayForm()
	--self.wndDisplay:Show(false, true)
	self.wndMain:Show(not self.wndMain:IsShown())
end

-- TODO: Verify this works without the following if necessary
function TextDisplay:UpdateUI() 
	self.wndMain:FindChild("EditBox"):SetText(self.settings.text[self.display_index])
	if self.settings.index == 1 then
		self.deleteButton:Show(false)
	else
		self.deleteButton:Show(true)
	end
	if self.display_index == self.settings.index then
		self.nextButton:Show(false)
	else 
		self.nextButton:Show(true)
	end
	if self.display_index == 1 then
		self.previousButton:Show(false)
	else 
		self.previousButton:Show(true)
	end
end

function TextDisplay:OnDisplay(wndHandler, wndControl, eMouseButton)
	if self.is_displayed == 0 then
		self:Display()
	else 
		self:Hide()
	end
end

function TextDisplay:Display()
	self.wndDisplay:FindChild("EditBox"):SetText(self.wndMain:FindChild("EditBox"):GetText())
	self.wndDisplay:Invoke()
	self.is_displayed = 1
	self.wndMain:FindChild("DisplayButton"):SetText("Hide")
end

function TextDisplay:Hide()
	self.wndDisplay:Close()
	self.is_displayed = 0
	self.wndMain:FindChild("DisplayButton"):SetText("Display")
end

function TextDisplay:OnReport(wndHandler, wndControl, eMouseButton)
	self:OutputDisplay()
end

function TextDisplay:OnPrevious(wndHandler, wndControl, eMouseButton)
	if self.display_index > 1 then
		self.display_index = self.display_index - 1
	end
	self:Hide()
	self:UpdateUI()
end

function TextDisplay:OnNext(wndHandler, wndControl, eMouseButton)
	if self.display_index < self.settings.index then
		self.display_index = self.display_index + 1
	end
	self:Hide()
	self:UpdateUI()
end

function TextDisplay:OnCreate(wndHandler, wndControl, eMouseButton)
	self.settings.index = self.settings.index + 1
	self.display_index = self.settings.index
	table.insert(self.settings.text, "Text example.")
	self:Hide()
	self:UpdateUI()
end

function TextDisplay:OnDelete(wndHandler, wndControl, eMouseButton)
	if self.settings.index > 1 then
		self.settings.index = self.settings.index - 1
		table.remove(self.settings.text, self.display_index)
		if self.display_index > self.settings.index then
			self.display_index = self.display_index - 1
		end
		self:Hide()
		self:UpdateUI()
	end
end

function TextDisplay:OnClose(wndHandler, wndControl, eMouseButton)
	self.wndMain:Show(false)
end

-----------------------------------------------------------------------------------------------
-- TextDisplay Instance
-----------------------------------------------------------------------------------------------
local TextDisplayInst = TextDisplay:new()
TextDisplayInst:Init()
